%###############    BASIC  #######################



:- module(basic, [
		  is_propVar/1,
		  is_prime/1,
		  is_disj/1,
		  is_primeOrDisj/1,
	          is_rposSubf/2,
	          get_rposSubf_form/2,
	          get_rposSubf_list/2,
		  tr_formula/2,
		  tr_formula_list/2,
		  pitpToProver/2,
	          contains_rule/2,
		  replace_leaves/3,
	          weak_tree_form/3,
	          weak_tree_set/3
	      ]
	   ).

:- use_module(syntax).
%:-use_module(printUtil,[
%	     print_error/1
%	     ]).


%% isPropVar(+A)
%
%   Check if A is a propositional variable



is_propVar(A) :- atom(A).


% is_Prime(+A)
%
%  Check if A is prime, namely:
%
%   A ::=  PropVar | #f

is_prime(A) :- A = #f ;   is_propVar(A).

% is_disj(A)
%
%  Check if A  is a disjunction A0 V A1

is_disj(_A0 v _A1).


% is_PrimecOrDisj(+A)
%
%  Check if A is prime or a disjunction, namely:
%
%  A = PropVar | #f  | A0 v A1

is_primeOrDisj(A) :- is_prime(A) ; is_disj(A).



% is_rposSubf(?PF,+A)
%
%  PF is a positive subformula of A


is_rposSubf(PF,PF).
is_rposSubf(PF, A & B) :- is_rposSubf(PF,A) ;  is_rposSubf(PF,B).
is_rposSubf(PF, _A -->  B) :- is_rposSubf(PF,B).

% get_rposSubf_form(+A,-PosSf)
%
% PosSubf = { PF | PF is a positive subformula of A }

get_rposSubf_form(A,PosSf) :-
	setof( PF, is_rposSubf(PF,A), PosSf).

% get_rposSubf_list(+Gamma,-PosSf)
%
% PosSubf = { PF | PF is a positive subformula of o formula A in Gamma }

get_rposSubf_list([],[]).
get_rposSubf_list([ A | Gamma],PosSf) :-
        get_rposSubf_form(A,PosSfA),
	get_rposSubf_list(Gamma,PosSf1),
        union(PosSfA,PosSf1,PosSf).



% tr_formula(+A,-F)
%
% Translate the formula A eliminating the defined operators



tr_formula(At,At) :- is_prime(At), !.


tr_formula(~A , F --> #f  ) :- !,
	tr_formula(A, F).


tr_formula(A0 & A1, F0 & F1) :- !,
	tr_formula(A0,F0),
	tr_formula(A1,F1).


tr_formula(A0 v A1, F0 v F1) :- !,
	tr_formula(A0,F0),
	tr_formula(A1,F1).

tr_formula(A0 --> A1, F0 --> F1) :- !,
	tr_formula(A0,F0),
	tr_formula(A1,F1).

tr_formula(A0 <--> A1, (F0 --> F1) & (F1 --> F0) ) :- !,
	tr_formula(A0,F0),
	tr_formula(A1,F1).

% ERROR

tr_formula(A, _) :-
        writeln('*****  Not a valid formula (check the syntax):\n'),
	writeln(A),
	abort.


% tr_formula_list(+Gamma,-Gamma1)
%
% Translate all the formulas in Gamma eliminating the defined operators

tr_formula_list([],[]).
tr_formula_list([A|AList],[F|FList]) :-
	tr_formula(A,F),
	tr_formula_list(AList,FList).






%%  translation from pitp syntax to this syntaxt

pitpToProver(non(A) , A1 -->  #f) :- !,
	pitpToProver(A,A1).

pitpToProver(and(A,B) , A1 & B1):- !,
	pitpToProver(A,A1),pitpToProver(B,B1).

pitpToProver(or(A,B) , A1 v B1) :- !,
	pitpToProver(A,A1),pitpToProver(B,B1).

pitpToProver(im(A,B) , A1 -->  B1):- !,
	pitpToProver(A,A1),pitpToProver(B,B1).

pitpToProver(equiv(A,B) , ( A1 --> B1) & (B1 --> A1) ):- !,
	pitpToProver(A,A1),pitpToProver(B,B1).

pitpToProver(A,A).


%% %  PROOFS



%% % contains_rule(+Rule,+Tree)
% true iff Tree contains an application of Rule

contains_rule(Rule,proof(Rule,_,_)) :- !.

contains_rule(Rule, proof(_,_,Prems)) :-
	      contains_rule_list(Rule,Prems).


contains_rule_list(Rule,[Tree | Trees]) :-
	contains_rule(Rule,Tree), !
	;
	contains_rule_list(Rule,Trees).



% replace_leaves(+P0,+P1,-Proof_out) :-
% Proof_out is obtained by replacing every open leaf sigma_0 of P1
%  with  P0
%
%		    P0
%		   sigma_0
%  Proof_out   =    P1
%
%
%
replace_leaves(P0, proof(leaf,_,[]), P0):- !.

replace_leaves(P0, proof(Rule,RootSeq,Prems),Proof_out) :-
        replace_leaves_list(P0,Prems,Prems_r),
        Proof_out =  proof(Rule,RootSeq,Prems_r).

replace_leaves_list(_P0,[],[]) :- !.

replace_leaves_list(P0, [Proof |Proofs] , [Proof_r|Proofs_r] ) :-
           replace_leaves(P0,Proof,Proof_r),
           replace_leaves_list(P0, Proofs , Proofs_r ).











%%% weak_tree_set(+MJC_Tree,+Gamma_add,-MJC_Tree_out)
%
% Tree_out is obtained from MJC_Tree by replacing every
% context Gamma occurring in   MJC_Tree with
%    Gamma U Gamma_add
%

weak_tree_set(proof(Rule, Gamma  ==> K , Prems), Gamma_add, Tree_out) :- !,
	union(Gamma,Gamma_add,Gamma1),
	weak_tree_set_list(Prems, Gamma_add , Prems1),
	Tree_out =  proof(Rule, Gamma1  ==> K , Prems1).

weak_tree_set(proof(Rule,Gamma @ foc(H) =*=> lab(L) @ F,Prems),Gamma_add,Tree_out) :- !,
	union(Gamma,Gamma_add,Gamma1),
	weak_tree_set_list(Prems, Gamma_add, Prems1),
	Tree_out = proof(Rule, Gamma1 @ foc(H) =*=> lab(L) @ F,Prems1).


weak_tree_set(proof(Rule,Gamma @ foc(H) '|>' ,Prems)  ,Gamma_add,Tree_out) :- !,
	union(Gamma,Gamma_add,Gamma1),
	weak_tree_set_list(Prems,Gamma_add , Prems1),
	Tree_out =   proof(Rule,  Gamma1 @ foc(H) '|>' ,  Prems1) .


weak_tree_set_list([], _Gamma , []):-!.

weak_tree_set_list( [Tree|Trees],Gamma, [Tree1| Trees1]) :-
	weak_tree_set(Tree,Gamma,Tree1),
	weak_tree_set_list(Trees,Gamma,Trees1).


%%% weak_tree_form(+MJC_Tree,+A,-MJC_Tree_out)
% As weak_tree_set, with the formula A added to the contexts


weak_tree_form(MJC_Tree,A,MJC_Tree_out) :-
	weak_tree_set(MJC_Tree,[A],MJC_Tree_out).



















